/*  -- translated by f2c (version 20201020 (for_lapack)). -- */

#include "f2c.h"

//> \brief \b DTRMM
//
// =========== DOCUMENTATION ===========
//
// Online html documentation available at
//           http://www.netlib.org/lapack/explore-html/
//
// Definition:
// ===========
//
//      SUBROUTINE DTRMM(SIDE,UPLO,TRANSA,DIAG,M,N,ALPHA,A,LDA,B,LDB)
//
//      .. Scalar Arguments ..
//      DOUBLE PRECISION ALPHA
//      INTEGER LDA,LDB,M,N
//      CHARACTER DIAG,SIDE,TRANSA,UPLO
//      ..
//      .. Array Arguments ..
//      DOUBLE PRECISION A(LDA,*),B(LDB,*)
//      ..
//
//
//> \par Purpose:
// =============
//>
//> \verbatim
//>
//> DTRMM  performs one of the matrix-matrix operations
//>
//>    B := alpha*op( A )*B,   or   B := alpha*B*op( A ),
//>
//> where  alpha  is a scalar,  B  is an m by n matrix,  A  is a unit, or
//> non-unit,  upper or lower triangular matrix  and  op( A )  is one  of
//>
//>    op( A ) = A   or   op( A ) = A**T.
//> \endverbatim
//
// Arguments:
// ==========
//
//> \param[in] SIDE
//> \verbatim
//>          SIDE is CHARACTER*1
//>           On entry,  SIDE specifies whether  op( A ) multiplies B from
//>           the left or right as follows:
//>
//>              SIDE = 'L' or 'l'   B := alpha*op( A )*B.
//>
//>              SIDE = 'R' or 'r'   B := alpha*B*op( A ).
//> \endverbatim
//>
//> \param[in] UPLO
//> \verbatim
//>          UPLO is CHARACTER*1
//>           On entry, UPLO specifies whether the matrix A is an upper or
//>           lower triangular matrix as follows:
//>
//>              UPLO = 'U' or 'u'   A is an upper triangular matrix.
//>
//>              UPLO = 'L' or 'l'   A is a lower triangular matrix.
//> \endverbatim
//>
//> \param[in] TRANSA
//> \verbatim
//>          TRANSA is CHARACTER*1
//>           On entry, TRANSA specifies the form of op( A ) to be used in
//>           the matrix multiplication as follows:
//>
//>              TRANSA = 'N' or 'n'   op( A ) = A.
//>
//>              TRANSA = 'T' or 't'   op( A ) = A**T.
//>
//>              TRANSA = 'C' or 'c'   op( A ) = A**T.
//> \endverbatim
//>
//> \param[in] DIAG
//> \verbatim
//>          DIAG is CHARACTER*1
//>           On entry, DIAG specifies whether or not A is unit triangular
//>           as follows:
//>
//>              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
//>
//>              DIAG = 'N' or 'n'   A is not assumed to be unit
//>                                  triangular.
//> \endverbatim
//>
//> \param[in] M
//> \verbatim
//>          M is INTEGER
//>           On entry, M specifies the number of rows of B. M must be at
//>           least zero.
//> \endverbatim
//>
//> \param[in] N
//> \verbatim
//>          N is INTEGER
//>           On entry, N specifies the number of columns of B.  N must be
//>           at least zero.
//> \endverbatim
//>
//> \param[in] ALPHA
//> \verbatim
//>          ALPHA is DOUBLE PRECISION.
//>           On entry,  ALPHA specifies the scalar  alpha. When  alpha is
//>           zero then  A is not referenced and  B need not be set before
//>           entry.
//> \endverbatim
//>
//> \param[in] A
//> \verbatim
//>           A is DOUBLE PRECISION array, dimension ( LDA, k ), where k is m
//>           when  SIDE = 'L' or 'l'  and is  n  when  SIDE = 'R' or 'r'.
//>           Before entry  with  UPLO = 'U' or 'u',  the  leading  k by k
//>           upper triangular part of the array  A must contain the upper
//>           triangular matrix  and the strictly lower triangular part of
//>           A is not referenced.
//>           Before entry  with  UPLO = 'L' or 'l',  the  leading  k by k
//>           lower triangular part of the array  A must contain the lower
//>           triangular matrix  and the strictly upper triangular part of
//>           A is not referenced.
//>           Note that when  DIAG = 'U' or 'u',  the diagonal elements of
//>           A  are not referenced either,  but are assumed to be  unity.
//> \endverbatim
//>
//> \param[in] LDA
//> \verbatim
//>          LDA is INTEGER
//>           On entry, LDA specifies the first dimension of A as declared
//>           in the calling (sub) program.  When  SIDE = 'L' or 'l'  then
//>           LDA  must be at least  max( 1, m ),  when  SIDE = 'R' or 'r'
//>           then LDA must be at least max( 1, n ).
//> \endverbatim
//>
//> \param[in,out] B
//> \verbatim
//>          B is DOUBLE PRECISION array, dimension ( LDB, N )
//>           Before entry,  the leading  m by n part of the array  B must
//>           contain the matrix  B,  and  on exit  is overwritten  by the
//>           transformed matrix.
//> \endverbatim
//>
//> \param[in] LDB
//> \verbatim
//>          LDB is INTEGER
//>           On entry, LDB specifies the first dimension of B as declared
//>           in  the  calling  (sub)  program.   LDB  must  be  at  least
//>           max( 1, m ).
//> \endverbatim
//
// Authors:
// ========
//
//> \author Univ. of Tennessee
//> \author Univ. of California Berkeley
//> \author Univ. of Colorado Denver
//> \author NAG Ltd.
//
//> \date December 2016
//
//> \ingroup double_blas_level3
//
//> \par Further Details:
// =====================
//>
//> \verbatim
//>
//>  Level 3 Blas routine.
//>
//>  -- Written on 8-February-1989.
//>     Jack Dongarra, Argonne National Laboratory.
//>     Iain Duff, AERE Harwell.
//>     Jeremy Du Croz, Numerical Algorithms Group Ltd.
//>     Sven Hammarling, Numerical Algorithms Group Ltd.
//> \endverbatim
//>
// =====================================================================
/* Subroutine */ int dtrmm_(char *side, char *uplo, char *transa, char *diag,
	int *m, int *n, double *alpha, double *a, int *lda, double *b, int *
	ldb)
{
    // System generated locals
    int a_dim1, a_offset, b_dim1, b_offset, i__1, i__2, i__3;

    // Local variables
    int i__, j, k, info;
    double temp;
    int lside;
    extern int lsame_(char *, char *);
    int nrowa;
    int upper;
    extern /* Subroutine */ int xerbla_(char *, int *);
    int nounit;

    //
    // -- Reference BLAS level3 routine (version 3.7.0) --
    // -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    // -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    //    December 2016
    //
    //    .. Scalar Arguments ..
    //    ..
    //    .. Array Arguments ..
    //    ..
    //
    // =====================================================================
    //
    //    .. External Functions ..
    //    ..
    //    .. External Subroutines ..
    //    ..
    //    .. Intrinsic Functions ..
    //    ..
    //    .. Local Scalars ..
    //    ..
    //    .. Parameters ..
    //    ..
    //
    //    Test the input parameters.
    //
    // Parameter adjustments
    a_dim1 = *lda;
    a_offset = 1 + a_dim1;
    a -= a_offset;
    b_dim1 = *ldb;
    b_offset = 1 + b_dim1;
    b -= b_offset;

    // Function Body
    lside = lsame_(side, "L");
    if (lside) {
	nrowa = *m;
    } else {
	nrowa = *n;
    }
    nounit = lsame_(diag, "N");
    upper = lsame_(uplo, "U");
    info = 0;
    if (! lside && ! lsame_(side, "R")) {
	info = 1;
    } else if (! upper && ! lsame_(uplo, "L")) {
	info = 2;
    } else if (! lsame_(transa, "N") && ! lsame_(transa, "T") && ! lsame_(
	    transa, "C")) {
	info = 3;
    } else if (! lsame_(diag, "U") && ! lsame_(diag, "N")) {
	info = 4;
    } else if (*m < 0) {
	info = 5;
    } else if (*n < 0) {
	info = 6;
    } else if (*lda < max(1,nrowa)) {
	info = 9;
    } else if (*ldb < max(1,*m)) {
	info = 11;
    }
    if (info != 0) {
	xerbla_("DTRMM ", &info);
	return 0;
    }
    //
    //    Quick return if possible.
    //
    if (*m == 0 || *n == 0) {
	return 0;
    }
    //
    //    And when  alpha.eq.zero.
    //
    if (*alpha == 0.) {
	i__1 = *n;
	for (j = 1; j <= i__1; ++j) {
	    i__2 = *m;
	    for (i__ = 1; i__ <= i__2; ++i__) {
		b[i__ + j * b_dim1] = 0.;
// L10:
	    }
// L20:
	}
	return 0;
    }
    //
    //    Start the operations.
    //
    if (lside) {
	if (lsame_(transa, "N")) {
	    //
	    //          Form  B := alpha*A*B.
	    //
	    if (upper) {
		i__1 = *n;
		for (j = 1; j <= i__1; ++j) {
		    i__2 = *m;
		    for (k = 1; k <= i__2; ++k) {
			if (b[k + j * b_dim1] != 0.) {
			    temp = *alpha * b[k + j * b_dim1];
			    i__3 = k - 1;
			    for (i__ = 1; i__ <= i__3; ++i__) {
				b[i__ + j * b_dim1] += temp * a[i__ + k *
					a_dim1];
// L30:
			    }
			    if (nounit) {
				temp *= a[k + k * a_dim1];
			    }
			    b[k + j * b_dim1] = temp;
			}
// L40:
		    }
// L50:
		}
	    } else {
		i__1 = *n;
		for (j = 1; j <= i__1; ++j) {
		    for (k = *m; k >= 1; --k) {
			if (b[k + j * b_dim1] != 0.) {
			    temp = *alpha * b[k + j * b_dim1];
			    b[k + j * b_dim1] = temp;
			    if (nounit) {
				b[k + j * b_dim1] *= a[k + k * a_dim1];
			    }
			    i__2 = *m;
			    for (i__ = k + 1; i__ <= i__2; ++i__) {
				b[i__ + j * b_dim1] += temp * a[i__ + k *
					a_dim1];
// L60:
			    }
			}
// L70:
		    }
// L80:
		}
	    }
	} else {
	    //
	    //          Form  B := alpha*A**T*B.
	    //
	    if (upper) {
		i__1 = *n;
		for (j = 1; j <= i__1; ++j) {
		    for (i__ = *m; i__ >= 1; --i__) {
			temp = b[i__ + j * b_dim1];
			if (nounit) {
			    temp *= a[i__ + i__ * a_dim1];
			}
			i__2 = i__ - 1;
			for (k = 1; k <= i__2; ++k) {
			    temp += a[k + i__ * a_dim1] * b[k + j * b_dim1];
// L90:
			}
			b[i__ + j * b_dim1] = *alpha * temp;
// L100:
		    }
// L110:
		}
	    } else {
		i__1 = *n;
		for (j = 1; j <= i__1; ++j) {
		    i__2 = *m;
		    for (i__ = 1; i__ <= i__2; ++i__) {
			temp = b[i__ + j * b_dim1];
			if (nounit) {
			    temp *= a[i__ + i__ * a_dim1];
			}
			i__3 = *m;
			for (k = i__ + 1; k <= i__3; ++k) {
			    temp += a[k + i__ * a_dim1] * b[k + j * b_dim1];
// L120:
			}
			b[i__ + j * b_dim1] = *alpha * temp;
// L130:
		    }
// L140:
		}
	    }
	}
    } else {
	if (lsame_(transa, "N")) {
	    //
	    //          Form  B := alpha*B*A.
	    //
	    if (upper) {
		for (j = *n; j >= 1; --j) {
		    temp = *alpha;
		    if (nounit) {
			temp *= a[j + j * a_dim1];
		    }
		    i__1 = *m;
		    for (i__ = 1; i__ <= i__1; ++i__) {
			b[i__ + j * b_dim1] = temp * b[i__ + j * b_dim1];
// L150:
		    }
		    i__1 = j - 1;
		    for (k = 1; k <= i__1; ++k) {
			if (a[k + j * a_dim1] != 0.) {
			    temp = *alpha * a[k + j * a_dim1];
			    i__2 = *m;
			    for (i__ = 1; i__ <= i__2; ++i__) {
				b[i__ + j * b_dim1] += temp * b[i__ + k *
					b_dim1];
// L160:
			    }
			}
// L170:
		    }
// L180:
		}
	    } else {
		i__1 = *n;
		for (j = 1; j <= i__1; ++j) {
		    temp = *alpha;
		    if (nounit) {
			temp *= a[j + j * a_dim1];
		    }
		    i__2 = *m;
		    for (i__ = 1; i__ <= i__2; ++i__) {
			b[i__ + j * b_dim1] = temp * b[i__ + j * b_dim1];
// L190:
		    }
		    i__2 = *n;
		    for (k = j + 1; k <= i__2; ++k) {
			if (a[k + j * a_dim1] != 0.) {
			    temp = *alpha * a[k + j * a_dim1];
			    i__3 = *m;
			    for (i__ = 1; i__ <= i__3; ++i__) {
				b[i__ + j * b_dim1] += temp * b[i__ + k *
					b_dim1];
// L200:
			    }
			}
// L210:
		    }
// L220:
		}
	    }
	} else {
	    //
	    //          Form  B := alpha*B*A**T.
	    //
	    if (upper) {
		i__1 = *n;
		for (k = 1; k <= i__1; ++k) {
		    i__2 = k - 1;
		    for (j = 1; j <= i__2; ++j) {
			if (a[j + k * a_dim1] != 0.) {
			    temp = *alpha * a[j + k * a_dim1];
			    i__3 = *m;
			    for (i__ = 1; i__ <= i__3; ++i__) {
				b[i__ + j * b_dim1] += temp * b[i__ + k *
					b_dim1];
// L230:
			    }
			}
// L240:
		    }
		    temp = *alpha;
		    if (nounit) {
			temp *= a[k + k * a_dim1];
		    }
		    if (temp != 1.) {
			i__2 = *m;
			for (i__ = 1; i__ <= i__2; ++i__) {
			    b[i__ + k * b_dim1] = temp * b[i__ + k * b_dim1];
// L250:
			}
		    }
// L260:
		}
	    } else {
		for (k = *n; k >= 1; --k) {
		    i__1 = *n;
		    for (j = k + 1; j <= i__1; ++j) {
			if (a[j + k * a_dim1] != 0.) {
			    temp = *alpha * a[j + k * a_dim1];
			    i__2 = *m;
			    for (i__ = 1; i__ <= i__2; ++i__) {
				b[i__ + j * b_dim1] += temp * b[i__ + k *
					b_dim1];
// L270:
			    }
			}
// L280:
		    }
		    temp = *alpha;
		    if (nounit) {
			temp *= a[k + k * a_dim1];
		    }
		    if (temp != 1.) {
			i__1 = *m;
			for (i__ = 1; i__ <= i__1; ++i__) {
			    b[i__ + k * b_dim1] = temp * b[i__ + k * b_dim1];
// L290:
			}
		    }
// L300:
		}
	    }
	}
    }
    return 0;
    //
    //    End of DTRMM .
    //
} // dtrmm_

